@@ -66,27 +66,30 @@ class VirtualBMC(bmc.Bmc):
                                            'bootdev': bootdevice})
         device = SET_BOOT_DEVICES_MAP.get(bootdevice)
         if device is None:
-            return 0xd5
+            # Invalid data field in request
+            return 0xcc
 
-        with utils.libvirt_open(**self._conn_args) as conn:
-            domain = utils.get_libvirt_domain(conn, self.domain_name)
-            tree = ET.fromstring(domain.XMLDesc())
+        try:
+            with utils.libvirt_open(**self._conn_args) as conn:
+                domain = utils.get_libvirt_domain(conn, self.domain_name)
+                tree = ET.fromstring(domain.XMLDesc())
 
-            for os_element in tree.findall('os'):
-                # Remove all "boot" elements
-                for boot_element in os_element.findall('boot'):
-                    os_element.remove(boot_element)
+                for os_element in tree.findall('os'):
+                    # Remove all "boot" elements
+                    for boot_element in os_element.findall('boot'):
+                        os_element.remove(boot_element)
 
-                # Add a new boot element with the request boot device
-                boot_element = ET.SubElement(os_element, 'boot')
-                boot_element.set('dev', device)
+                    # Add a new boot element with the request boot device
+                    boot_element = ET.SubElement(os_element, 'boot')
+                    boot_element.set('dev', device)
 
-            try:
                 conn.defineXML(ET.tostring(tree))
-            except libvirt.libvirtError:
-                LOG.error('Failed setting the boot device  %(bootdev)s for '
-                          'domain %(domain)s', {'bootdev': device,
-                                                'domain': self.domain_name})
+        except libvirt.libvirtError:
+            LOG.error('Failed setting the boot device  %(bootdev)s for '
+                      'domain %(domain)s', {'bootdev': device,
+                                            'domain': self.domain_name})
+            # Command not supported in present state
+            return 0xd5
 
     def get_power_state(self):
         LOG.debug('Get power state called for domain %s', self.domain_name)
@@ -99,10 +102,25 @@ class VirtualBMC(bmc.Bmc):
             LOG.error('Error getting the power state of domain %(domain)s. '
                       'Error: %(error)s', {'domain': self.domain_name,
                                            'error': e})
-            return
+            # Command not supported in present state
+            return 0xd5
 
         return POWEROFF
 
+    def pulse_diag(self):
+        LOG.debug('Power diag called for domain %s', self.domain_name)
+        try:
+            with utils.libvirt_open(**self._conn_args) as conn:
+                domain = utils.get_libvirt_domain(conn, self.domain_name)
+                if domain.isActive():
+                    domain.injectNMI()
+        except libvirt.libvirtError as e:
+            LOG.error('Error powering diag the domain %(domain)s. '
+                      'Error: %(error)s' % {'domain': self.domain_name,
+                                            'error': e})
+            # Command not supported in present state
+            return 0xd5
+
     def power_off(self):
         LOG.debug('Power off called for domain %s', self.domain_name)
         try:
@@ -114,7 +132,8 @@ class VirtualBMC(bmc.Bmc):
             LOG.error('Error powering off the domain %(domain)s. '
                       'Error: %(error)s' % {'domain': self.domain_name,
                                             'error': e})
-            return
+            # Command not supported in present state
+            return 0xd5
 
     def power_on(self):
         LOG.debug('Power on called for domain %s', self.domain_name)
@@ -127,4 +146,19 @@ class VirtualBMC(bmc.Bmc):
             LOG.error('Error powering on the domain %(domain)s. '
                       'Error: %(error)s' % {'domain': self.domain_name,
                                             'error': e})
-            return
+            # Command not supported in present state
+            return 0xd5
+
+    def power_shutdown(self):
+        LOG.debug('Soft power off called for domain %s', self.domain_name)
+        try:
+            with utils.libvirt_open(**self._conn_args) as conn:
+                domain = utils.get_libvirt_domain(conn, self.domain_name)
+                if domain.isActive():
+                    domain.shutdown()
+        except libvirt.libvirtError as e:
+            LOG.error('Error soft powering off the domain %(domain)s. '
+                      'Error: %(error)s' % {'domain': self.domain_name,
+                                            'error': e})
+            # Command not supported in present state
+            return 0xd5
